package bank3tier.lg;

/** Generic Transaction delivering a result of type RESULT.
 * @author Christoph Knabe, Beuth University of Applied Sciences, Berlin, Germany
 * @since 2014-07-14
*/
public abstract class LgTransaction<RESULT> {

	private final RESULT _result;

	/** Creates a transaction in the session lgSitzung and executes the method execute() in it.
	 * The result of the method execute is afterwards accessible by the method getResult().
	 * This constructor follows the Template Method Pattern in order to guarantee,
	 * that the transaction is correctly terminated either successfully or erroneous.
	 * If execute() returns successfully, then lgSitzung.endTransaction(true) is being called,
	 * if execute() terminates with an exception, then lgSitzung.endTransaction(false) is being called.
	*/
	public LgTransaction(final LgSitzung lgSitzung) throws Exception {
		boolean ok = false;
		lgSitzung.beginTransaction();
		try{
			_result = execute();
			ok = true;
		}finally{
			lgSitzung.endTransaction(ok);
		}
	}

	/** Returns the result of a successful execution of the method execute().
	 * The result is presumably null, if the method execute() terminated with an exception.
	*/
	public final RESULT getResult(){
		return _result;
	}

	/** Override this method with the behavior, that should be executed inside a transaction. */
	protected abstract RESULT execute() throws Exception;

}
